@@ -1,12 +1,12 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
-# 
+#
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 #    http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -25,28 +25,28 @@ from mapproxy.util.ext.local import LocalStack
 class Options(dict):
     """
     Dictionary with attribute style access.
-    
+
     >>> o = Options(bar='foo')
     >>> o.bar
     'foo'
     """
     def __repr__(self):
         return '%s(%s)' % (self.__class__.__name__, dict.__repr__(self))
-    
+
     def __getattr__(self, name):
         if name in self:
             return self[name]
         else:
             raise AttributeError(name)
-    
+
     __setattr__ = dict.__setitem__
-    
+
     def __delattr__(self, name):
         if name in self:
             del self[name]
         else:
             raise AttributeError(name)
-    
+
     def update(self, other=None, **kw):
         if other is not None:
             if hasattr(other, 'iteritems'):
@@ -60,7 +60,7 @@ class Options(dict):
                 self[key].update(value)
             else:
                 self[key] = value
-    
+
     def __deepcopy__(self, memo):
         return Options(copy.deepcopy(self.items(), memo))
 
@@ -93,10 +93,13 @@ def _to_options_map(mapping):
     else:
         return mapping
 
-def abspath(path):
+def abspath(path, base_path=None):
     """
-    Convert path to absolute path. Uses ``conf_base_dir`` as base, if path is relative. 
+    Convert path to absolute path. Uses ``conf_base_dir`` as base, if
+    path is relative and ``base_path`` is not set.
     """
+    if base_path:
+        return os.path.abspath(os.path.join(base_path, path))
     return os.path.join(base_config().conf_base_dir, path)
 
 
@@ -113,10 +116,10 @@ def finish_base_config(bc=None):
         bc.srs.axis_order_en = default_en.union(set(bc.srs.axis_order_en))
         if 'proj_data_dir' in bc.srs:
             bc.srs.proj_data_dir = os.path.join(bc.conf_base_dir, bc.srs.proj_data_dir)
-        
+
     if 'wms' in bc:
         bc.wms.srs = set(bc.wms.srs)
-    
+
     if 'conf_base_dir' in bc:
         if 'cache' in bc:
             if 'base_dir' in bc.cache:
@@ -127,13 +130,13 @@ def finish_base_config(bc=None):
 def load_base_config(config_file=None, clear_existing=False):
     """
     Load system wide base configuration.
-    
+
     :param config_file: the file name of the mapproxy.yaml configuration.
                         if ``None``, load the internal proxylib/default.yaml conf
     :param clear_existing: if ``True`` remove the existing configuration settings,
                            else overwrite the settings.
     """
-    
+
     if config_file is None:
         from mapproxy.config import defaults
         config_dict = {}
@@ -145,10 +148,10 @@ def load_base_config(config_file=None, clear_existing=False):
     else:
         conf_base_dir = os.path.abspath(os.path.dirname(config_file))
         load_config(base_config(), config_file=config_file, clear_existing=clear_existing)
-    
+
     bc = base_config()
     finish_base_config(bc)
-    
+
     bc.conf_base_dir = conf_base_dir
 
 def load_default_config():
@@ -165,13 +168,13 @@ def load_default_config():
 def load_config(config, config_file=None, config_dict=None, clear_existing=False):
     if clear_existing:
         for key in config.keys():
-            del config[key] 
-    
+            del config[key]
+
     if config_dict is None:
         config_dict = load_yaml_file(config_file)
-    
+
     defaults = _to_options_map(config_dict)
-    
+
     if defaults:
         for key, value in defaults.iteritems():
             if key in config and hasattr(config[key], 'update'):
