@@ -1,12 +1,12 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010, 2011 Omniscale <http://omniscale.de>
-# 
+#
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 #    http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,13 +15,19 @@
 
 from __future__ import with_statement, division
 
+import os
 import sys
+import stat
 import math
 import time
+import cPickle as pickle
 from datetime import datetime
 
 from mapproxy.layer import map_extent_from_grid
 
+import logging
+log = logging.getLogger(__name__)
+
 class bidict(dict):
     """
     Simplest bi-directional dictionary.
@@ -49,8 +55,9 @@ class ETA(object):
 
             while missing_ticks > 0:
 
-                self.tick_duration_sums *= 0.99
-                self.tick_duration_divisor *= 0.99
+                # reduce the influence of older messurements
+                self.tick_duration_sums *= 0.999
+                self.tick_duration_divisor *= 0.999
 
                 self.tick_count += 1
 
@@ -65,45 +72,107 @@ class ETA(object):
         timestamp = self.eta()
         if timestamp is None:
             return 'N/A'
-        return time.strftime('%Y-%m-%d-%H:%M:%S', time.localtime(timestamp))
+        try:
+            return time.strftime('%Y-%m-%d-%H:%M:%S', time.localtime(timestamp))
+        except ValueError:
+            # raised when time is out of range (e.g. year >2038)
+            return 'N/A'
 
     def eta(self):
         if not self.tick_count: return
-        return (self.last_tick_start + 
-                (1-self.progress) * (self.tick_duration_sums/self.tick_duration_divisor) * self.ticks)
+        return (self.last_tick_start +
+                ((self.tick_duration_sums/self.tick_duration_divisor)
+                 * (self.ticks - self.tick_count)))
 
     def __str__(self):
         return self.eta_string()
 
+class ProgressStore(object):
+    """
+    Reads and stores seed progresses to a file.
+    """
+    def __init__(self, filename=None, continue_seed=True):
+        self.filename = filename
+        if continue_seed:
+            self.status = self.load()
+        else:
+            self.status = {}
+
+    def load(self):
+        if not os.path.exists(self.filename):
+            pass
+        elif os.stat(self.filename).st_mode & stat.S_IWOTH:
+            log.error('progress file (%s) is world writable, ignoring file',
+                self.filename)
+        else:
+            with open(self.filename) as f:
+                try:
+                    return pickle.load(f)
+                except (pickle.UnpicklingError, AttributeError,
+                    EOFError, ImportError, IndexError):
+                    log.error('unable to read progress file (%s), ignoring file',
+                        self.filename)
+
+        return {}
+
+    def write(self):
+        try:
+            with open(self.filename + '.tmp', 'w') as f:
+                f.write(pickle.dumps(self.status))
+                f.flush()
+                os.fsync(f.fileno())
+            os.rename(self.filename + '.tmp', self.filename)
+        except (IOError, OSError), ex:
+            log.error('unable to write seed progress: %s', ex)
+
+    def remove(self):
+        self.status = {}
+        if os.path.exists(self.filename):
+            os.remove(self.filename)
+
+    def get(self, task_identifier):
+        return self.status.get(task_identifier, None)
+
+    def add(self, task_identifier, progress_identifier):
+        self.status[task_identifier] = progress_identifier
+
 class ProgressLog(object):
-    def __init__(self, out=None, silent=False, verbose=True):
+    def __init__(self, out=None, silent=False, verbose=True, progress_store=None):
         if not out:
             out = sys.stdout
         self.out = out
         self.lastlog = time.time()
         self.verbose = verbose
         self.silent = silent
-        
+        self.current_task_id = None
+        self.progress_store = progress_store
+
     def log_step(self, progress):
         if not self.verbose:
             return
         if (self.lastlog + .1) < time.time():
             # log progress at most every 100ms
-            self.out.write('[%s] %6.2f%% %s \tETA: %s\r' % (
-                timestamp(), progress[1]*100, progress[0],
-                progress[2]
+            self.out.write('[%s] %6.2f%%\t%-20s ETA: %s\r' % (
+                timestamp(), progress.progress*100, progress.progress_str,
+                progress.eta
             ))
             self.out.flush()
             self.lastlog = time.time()
-    
-    def log_progress(self, progress, level, bbox, tiles, eta):
+
+    def log_progress(self, progress, level, bbox, tiles):
+        if self.progress_store and self.current_task_id:
+            self.progress_store.add(self.current_task_id,
+                progress.current_progress_identifier())
+            self.progress_store.write()
+
         if self.silent:
             return
         self.out.write('[%s] %2s %6.2f%% %s (%d tiles) ETA: %s\n' % (
-            timestamp(), level, progress*100,
-            format_bbox(bbox), tiles, eta))
+            timestamp(), level, progress.progress*100,
+            format_bbox(bbox), tiles, progress.eta))
         self.out.flush()
 
+
 def limit_sub_bbox(bbox, sub_bbox):
     """
     >>> limit_sub_bbox((0, 1, 10, 11), (-1, -1, 9, 8))
@@ -116,7 +185,7 @@ def limit_sub_bbox(bbox, sub_bbox):
     maxx = min(bbox[2], sub_bbox[2])
     maxy = min(bbox[3], sub_bbox[3])
     return minx, miny, maxx, maxy
-    
+
 def timestamp():
     return datetime.now().strftime('%H:%M:%S')
 
@@ -139,7 +208,7 @@ def status_symbol(i, total):
     else:
         return symbols[int(math.ceil(i/(total/4)))]
 
-def exp_backoff(func, args=(), kw={}, max_repeat=10, start_backoff_sec=2, 
+def exp_backoff(func, args=(), kw={}, max_repeat=10, start_backoff_sec=2,
         exceptions=(Exception,), ignore_exceptions=tuple()):
     n = 0
     while True:
@@ -151,7 +220,7 @@ def exp_backoff(func, args=(), kw={}, max_repeat=10, start_backoff_sec=2,
             if (n+1) >= max_repeat:
                 raise
             wait_for = start_backoff_sec * 2**n
-            print >>sys.stderr, ("An error occured. Retry in %d seconds: %r" % 
+            print >>sys.stderr, ("An error occured. Retry in %d seconds: %r" %
                 (wait_for, ex))
             time.sleep(wait_for)
             n += 1
@@ -168,7 +237,7 @@ def format_seed_task(task):
     else:
         info.append('   Complete grid: %s (EPSG:4326)' % (format_bbox(map_extent_from_grid(task.grid).llbbox), ))
     info.append('    Levels: %s' % (task.levels, ))
-        
+
     if task.refresh_timestamp:
         info.append('    Overwriting: tiles older than %s' %
                     datetime.fromtimestamp(task.refresh_timestamp))
@@ -176,7 +245,7 @@ def format_seed_task(task):
         info.append('    Overwriting: all tiles')
     else:
         info.append('    Overwriting: no tiles')
-    
+
     return '\n'.join(info)
 
 def format_cleanup_task(task):
@@ -189,11 +258,11 @@ def format_cleanup_task(task):
     else:
         info.append('    Complete grid: %s (EPSG:4326)' % (format_bbox(map_extent_from_grid(task.grid).llbbox), ))
     info.append('    Levels: %s' % (task.levels, ))
-        
+
     if task.remove_timestamp:
         info.append('    Remove: tiles older than %s' %
                     datetime.fromtimestamp(task.remove_timestamp))
     else:
         info.append('    Remove: all tiles')
-    
+
     return '\n'.join(info)
