@@ -1,12 +1,12 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2011 Omniscale <http://omniscale.de>
-# 
+#
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 #    http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -23,30 +23,35 @@ from mapproxy.request.wmts import wmts_request, make_wmts_rest_request_parser
 from mapproxy.service.base import Server
 from mapproxy.response import Response
 from mapproxy.exception import RequestError
+from mapproxy.util.coverage import load_limited_to
 
 from mapproxy.template import template_loader, bunch
 env = {'bunch': bunch}
-get_template = template_loader(__file__, 'templates', namespace=env)
+get_template = template_loader(__name__, 'templates', namespace=env)
 
 import logging
 log = logging.getLogger(__name__)
 
 class WMTSServer(Server):
     service = 'wmts'
-    
-    def __init__(self, layers, md, request_parser=None):
+
+    def __init__(self, layers, md, request_parser=None, max_tile_age=None):
         Server.__init__(self)
         self.request_parser = request_parser or wmts_request
         self.md = md
-        self.max_tile_age = None # TODO
+        self.max_tile_age = max_tile_age
         self.layers, self.matrix_sets = self._matrix_sets(layers)
         self.capabilities_class = Capabilities
-    
+
     def _matrix_sets(self, layers):
         sets = {}
         layers_grids = {}
         for layer in layers.values():
             grid = layer.grid
+            if not grid.supports_access_with_origin('nw'):
+                log.warn("grid '%s' is not compatible with WMTS, skipping for layer '%s'",
+                    grid.name, layer.name)
+                continue
             if grid.name not in sets:
                 try:
                     sets[grid.name] = TileMatrixSet(grid)
@@ -57,25 +62,65 @@ class WMTSServer(Server):
         for layer_name, layers in layers_grids.items():
             wmts_layers[layer_name] = WMTSTileLayer(layers)
         return wmts_layers, sets.values()
-        
+
     def capabilities(self, request):
         service = self._service_md(request)
-        result = self.capabilities_class(service, self.layers.values(), self.matrix_sets).render(request)
+        layers = self.authorized_tile_layers(request.http.environ)
+        result = self.capabilities_class(service, layers, self.matrix_sets).render(request)
         return Response(result, mimetype='application/xml')
-    
+
     def tile(self, request):
         self.check_request(request)
         tile_layer = self.layers[request.layer][request.tilematrixset]
         if not request.format:
             request.format = tile_layer.format
 
-        tile = tile_layer.render(request)
-        resp = Response(tile.as_buffer(), content_type='image/' + request.format)
+        limited_to = self.authorize_tile_layer(tile_layer, request)
+        tile = tile_layer.render(request, coverage=limited_to)
+
+        # set the content_type to tile.format and not to request.format ( to support mixed_mode)
+        resp = Response(tile.as_buffer(), content_type='image/' + tile.format)
         resp.cache_headers(tile.timestamp, etag_data=(tile.timestamp, tile.size),
                            max_age=self.max_tile_age)
         resp.make_conditional(request.http)
         return resp
-    
+
+    def authorize_tile_layer(self, tile_layer, request):
+        if 'mapproxy.authorize' in request.http.environ:
+            query_extent = tile_layer.grid.srs.srs_code, tile_layer.tile_bbox(request)
+            result = request.http.environ['mapproxy.authorize']('wmts', [request.layer],
+                query_extent=query_extent, environ=request.http.environ)
+            if result['authorized'] == 'unauthenticated':
+                raise RequestError('unauthorized', status=401)
+            if result['authorized'] == 'full':
+                return
+            if result['authorized'] == 'partial':
+                if result['layers'].get(request.layer, {}).get('tile', False) == True:
+                    limited_to = result.get('limited_to')
+                    if limited_to:
+                        return load_limited_to(limited_to)
+                    else:
+                        return None
+            raise RequestError('forbidden', status=403)
+
+    def authorized_tile_layers(self, env):
+        if 'mapproxy.authorize' in env:
+            result = env['mapproxy.authorize']('wmts', [l for l in self.layers],
+                query_extent=None, environ=env)
+            if result['authorized'] == 'unauthenticated':
+                raise RequestError('unauthorized', status=401)
+            if result['authorized'] == 'full':
+                return self.layers.values()
+            if result['authorized'] == 'none':
+                raise RequestError('forbidden', status=403)
+            allowed_layers = []
+            for layer in self.layers.itervalues():
+                if result['layers'].get(layer.name, {}).get('tile', False) == True:
+                    allowed_layers.append(layer)
+            return allowed_layers
+        else:
+            return self.layers.values()
+
     def check_request(self, request):
         request.make_tile_request()
         if request.layer not in self.layers:
@@ -93,20 +138,20 @@ class WMTSServer(Server):
 
 class WMTSRestServer(WMTSServer):
     """
-    OGC WMTS 1.0.0 RESTful Server 
+    OGC WMTS 1.0.0 RESTful Server
     """
     service = None
     names = ('wmts',)
     request_methods = ('tile', 'capabilities')
-    default_template = '/{{Layer}}/{{TileMatrixSet}}/{{TileMatrix}}/{{TileCol}}/{{TileRow}}.{{Format}}'
-    
+    default_template = '/{Layer}/{TileMatrixSet}/{TileMatrix}/{TileCol}/{TileRow}.{Format}'
+
     def __init__(self, layers, md, max_tile_age=None, template=None):
         WMTSServer.__init__(self, layers, md)
         self.max_tile_age = max_tile_age
         self.template = template or self.default_template
         self.request_parser = make_wmts_rest_request_parser(self.template)
         self.capabilities_class = partial(RestfulCapabilities, template=self.template)
-    
+
 
 class Capabilities(object):
     """
@@ -116,10 +161,10 @@ class Capabilities(object):
         self.service = server_md
         self.layers = layers
         self.matrix_sets = matrix_sets
-    
+
     def render(self, _map_request):
         return self._render_template(_map_request.capabilities_template)
-    
+
     def template_context(self):
         return dict(service=bunch(default='', **self.service),
                     restful=False,
@@ -137,7 +182,7 @@ class RestfulCapabilities(Capabilities):
     def __init__(self, server_md, layers, matrix_sets, template):
         Capabilities.__init__(self, server_md, layers, matrix_sets)
         self.template = template
-    
+
     def template_context(self):
         return dict(service=bunch(default='', **self.service),
                     restful=True,
@@ -148,9 +193,15 @@ class RestfulCapabilities(Capabilities):
                     )
 
 def format_resource_template(layer, template, service):
+    # TODO: remove {{Format}} in 1.6
     if '{{Format}}' in template:
         template = template.replace('{{Format}}', layer.format)
-    
+    if '{Format}' in template:
+        template = template.replace('{Format}', layer.format)
+
+    if '{Layer}' in template:
+        template = template.replace('{Layer}', layer.name)
+
     return service.url + template
 
 class WMTSTileLayer(object):
@@ -161,16 +212,16 @@ class WMTSTileLayer(object):
         self.grids = [lyr.grid for lyr in layers.values()]
         self.layers = layers
         self._layer = layers[layers.keys()[0]]
-    
+
     def __getattr__(self, name):
         return getattr(self._layer, name)
-    
+
     def __contains__(self, gridname):
         return gridname in self.layers
-    
+
     def __getitem__(self, gridname):
         return self.layers[gridname]
-    
+
 
 from mapproxy.grid import tile_grid
 
@@ -187,8 +238,12 @@ class TileMatrixSet(object):
         self.grid = grid
         self.name = grid.name
         self.srs_name = grid.srs.srs_code
-    
+        self.tile_matrices = list(self._tile_matrices())
+
     def __iter__(self):
+        return iter(self.tile_matrices)
+
+    def _tile_matrices(self):
         for level, res in self.grid.resolutions.iteritems():
             origin = self.grid.origin_tile(level, 'ul')
             bbox = self.grid.tile_bbox(origin)
