@@ -89,46 +89,20 @@ def SRS(srs_code):
         _thread_local.srs_cache[srs_code] = srs
         return srs
 
-def epsg900913():
-    try:
-        return Proj(proj_def='''PROJCS["Google Mercator", 
-       GEOGCS["WGS 84", 
-         DATUM["World Geodetic System 1984", 
-           SPHEROID["WGS 84", 6378137.0, 298.257223563, AUTHORITY["EPSG","7030"]], 
-           AUTHORITY["EPSG","6326"]], 
-         PRIMEM["Greenwich", 0.0, AUTHORITY["EPSG","8901"]], 
-         UNIT["degree", 0.017453292519943295], 
-         AXIS["Geodetic latitude", NORTH], 
-         AXIS["Geodetic longitude", EAST], 
-         AUTHORITY["EPSG","4326"]], 
-       PROJECTION["Mercator (1SP)", AUTHORITY["EPSG","9804"]], 
-       PARAMETER["semi_major", 6378137.0], 
-       PARAMETER["semi_minor", 6378137.0], 
-       PARAMETER["latitude_of_origin", 0.0], 
-       PARAMETER["central_meridian", 0.0], 
-       PARAMETER["scale_factor", 1.0], 
-       PARAMETER["false_easting", 0.0], 
-       PARAMETER["false_northing", 0.0], 
-       UNIT["m", 1.0], 
-       AXIS["Easting", EAST], 
-       AXIS["Northing", NORTH], 
-       AUTHORITY["EPSG","900913"]]''')
-    except RuntimeError:
-        return Proj('+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 '
-                '+lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m '
-                '+nadgrids=@null +no_defs')
-
 WEBMERCATOR_EPSG = set(('EPSG:900913', 'EPSG:3857',
     'EPSG:102100', 'EPSG:102113'))
 
 class _SRS(object):
     # http://trac.openlayers.org/wiki/SphericalMercator
-    # TODO jproj/libproj
     proj_init = {
-                 'CRS:84': lambda: Proj(init='epsg:4326'),
+                 'EPSG:4326': lambda: Proj('+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs +over'),
+                 'CRS:84': lambda: Proj('+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs +over'),
                 }
     for _epsg in WEBMERCATOR_EPSG:
-        proj_init[_epsg] = epsg900913
+        proj_init[_epsg] = lambda: Proj(
+            '+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 '
+            '+lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m '
+            '+nadgrids=@null +no_defs +over')
         
     """
     This class represents a Spatial Reference System.
@@ -139,15 +113,12 @@ class _SRS(object):
         """
         self.srs_code = srs_code
         
-        try:
+        init = _SRS.proj_init.get(srs_code, None)
+        if init is not None:
+            self.proj = init()
+        else:
             epsg_num = get_epsg_num(srs_code)   
             self.proj = Proj(init='epsg:%d' % epsg_num)
-        except (ProjInitError, RuntimeError), ex:
-            init = _SRS.proj_init.get(srs_code, None)
-            if init is not None:
-                self.proj = init()
-            else:
-                raise ex
     
     def transform_to(self, other_srs, points):
         """
@@ -396,12 +367,13 @@ def merge_bbox(bbox1, bbox2):
     maxy = max(bbox1[3], bbox2[3])
     return (minx, miny, maxx, maxy)
 
-def bbox_equals(src_bbox, dst_bbox, x_delta, y_delta=None):
+def bbox_equals(src_bbox, dst_bbox, x_delta=None, y_delta=None):
     """
     Compares two bbox and checks if they are equal, or nearly equal.
     
     :param x_delta: how precise the comparison should be.
-                    should be reasonable small, like a tenth of a pixle
+                    should be reasonable small, like a tenth of a pixel.
+                    defaults to 1/1.000.000th of the width.
     :type x_delta: bbox units
     
     >>> src_bbox = (939258.20356824622, 6887893.4928338043, 
@@ -413,6 +385,8 @@ def bbox_equals(src_bbox, dst_bbox, x_delta, y_delta=None):
     >>> bbox_equals(src_bbox, dst_bbox, 0.0001)
     False
     """
+    if x_delta is None:
+        x_delta = abs(src_bbox[0] - src_bbox[2]) / 1000000.0
     if y_delta is None:
         y_delta = x_delta
     return (abs(src_bbox[0] - dst_bbox[0]) < x_delta and
@@ -420,7 +394,6 @@ def bbox_equals(src_bbox, dst_bbox, x_delta, y_delta=None):
             abs(src_bbox[2] - dst_bbox[2]) < y_delta and
             abs(src_bbox[3] - dst_bbox[3]) < y_delta)
 
-
 def make_lin_transf(src_bbox, dst_bbox):
     """
     Create a transformation function that transforms linear between two
