@@ -43,11 +43,11 @@ class TestCoverageWMS(SystemTest):
     def test_capababilities(self):
         resp = self.app.get('/service?request=GetCapabilities&service=WMS&version=1.1.1')
         xml = resp.lxml
-        # First: combined root, second: wms_cache, third: tms_cache
-        eq_(xml.xpath('//LatLonBoundingBox/@minx'), ['10', '10', '12'])
-        eq_(xml.xpath('//LatLonBoundingBox/@miny'), ['10', '15', '10'])
-        eq_(xml.xpath('//LatLonBoundingBox/@maxx'), ['35', '30', '35'])
-        eq_(xml.xpath('//LatLonBoundingBox/@maxy'), ['31', '31', '30'])
+        # First: combined root, second: wms_cache, third: tms_cache, last: seed_only
+        eq_(xml.xpath('//LatLonBoundingBox/@minx'), ['10', '10', '12', '14'])
+        eq_(xml.xpath('//LatLonBoundingBox/@miny'), ['10', '15', '10', '13'])
+        eq_(xml.xpath('//LatLonBoundingBox/@maxx'), ['35', '30', '35', '24'])
+        eq_(xml.xpath('//LatLonBoundingBox/@maxy'), ['31', '31', '30', '23'])
 
     def test_get_map_outside(self):
         self.common_map_req.params.bbox = -90, 0, 0, 90
@@ -98,7 +98,7 @@ class TestCoverageTMS(SystemTest):
                                       '&REQUEST=GetMap&HEIGHT=24&SRS=EPSG%3A900913&styles='
                                       '&VERSION=1.1.1&BBOX=1113194.90793,1689200.13961,3339584.7238,3632749.14338'
                                       '&WIDTH=28'},
-                            {'body': img.read(), 'headers': {'content-type': 'image/jgeg'}})
+                            {'body': img.read(), 'headers': {'content-type': 'image/jpeg'}})
             with mock_httpd(('localhost', 42423), [expected_req]):
                 resp = self.app.get('/tms/1.0.0/wms_cache/0/1/1.jpeg')
                 eq_(resp.content_type, 'image/jpeg')
@@ -107,7 +107,7 @@ class TestCoverageTMS(SystemTest):
     def test_get_tile_intersection_tms(self):
         with tmp_image((256, 256), format='jpeg') as img:
             expected_req = ({'path': r'/tms/1.0.0/foo/1/1/1.jpeg'},
-                            {'body': img.read(), 'headers': {'content-type': 'image/jgeg'}})
+                            {'body': img.read(), 'headers': {'content-type': 'image/jpeg'}})
             with mock_httpd(('localhost', 42423), [expected_req]):
                 resp = self.app.get('/tms/1.0.0/tms_cache/0/1/1.jpeg')
                 eq_(resp.content_type, 'image/jpeg')
