@@ -1,12 +1,12 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2010 Omniscale <http://omniscale.de>
-# 
+#
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 #    http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -29,7 +29,7 @@ from mapproxy.request.wms import WMS100MapRequest, WMS111MapRequest, WMS130MapRe
                                  WMS110MapRequest, WMS110FeatureInfoRequest, \
                                  WMS110CapabilitiesRequest, \
                                  wms_request
-from mapproxy.test.image import is_jpeg, is_png, tmp_image
+from mapproxy.test.image import is_jpeg, is_png, tmp_image, create_tmp_image
 from mapproxy.test.http import mock_httpd
 from mapproxy.test.helper import validate_with_dtd, validate_with_xsd
 from nose.tools import eq_, assert_almost_equal
@@ -65,7 +65,7 @@ def is_111_exception(xml, msg=None, code=None, re_msg=None):
     if code is not None:
         eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code')[0], code)
     assert validate_with_dtd(xml, 'wms/1.1.1/exception_1_1_1.dtd')
-    
+
 def is_111_capa(xml):
     return validate_with_dtd(xml, dtd_name='wms/1.1.1/WMS_MS_Capabilities.dtd')
 def is_130_capa(xml):
@@ -76,7 +76,7 @@ class WMSTest(SystemTest):
     config = test_config
 
 class TestCoverageWMS(WMSTest):
-    
+
     def test_unknown_version_110(self):
         resp = self.app.get('http://localhost/service?SERVICE=WMS&REQUEST=GetCapabilities'
                             '&VERSION=1.1.0')
@@ -97,9 +97,9 @@ class TestCoverageWMS(WMSTest):
 class TestWMS111(WMSTest):
     def setup(self):
         WMSTest.setup(self)
-        self.common_req = WMS111MapRequest(url='/service?', param=dict(service='WMS', 
+        self.common_req = WMS111MapRequest(url='/service?', param=dict(service='WMS',
              version='1.1.1'))
-        self.common_map_req = WMS111MapRequest(url='/service?', param=dict(service='WMS', 
+        self.common_map_req = WMS111MapRequest(url='/service?', param=dict(service='WMS',
              version='1.1.1', bbox='-180,0,0,80', width='200', height='200',
              layers='wms_cache', srs='EPSG:4326', format='image/png',
              styles='', request='GetMap'))
@@ -107,12 +107,12 @@ class TestWMS111(WMSTest):
             param=dict(x='10', y='20', width='200', height='200', layers='wms_cache',
                        format='image/png', query_layers='wms_cache', styles='',
                        bbox='1000,400,2000,1400', srs='EPSG:900913'))
-    
+
     def test_invalid_request_type(self):
         req = str(self.common_map_req).replace('GetMap', 'invalid')
         resp = self.app.get(req)
         is_111_exception(resp.lxml, "unknown WMS request type 'invalid'")
-        
+
     def test_wms_capabilities(self):
         req = WMS111CapabilitiesRequest(url='/service?').copy_with_request_params(self.common_req)
         resp = self.app.get(req)
@@ -121,54 +121,63 @@ class TestWMS111(WMSTest):
         eq_(xml.xpath('//GetMap//OnlineResource/@xlink:href',
                       namespaces=dict(xlink="http://www.w3.org/1999/xlink"))[0],
             'http://localhost/service?')
+
+        # test for MetadataURL
+        eq_(xml.xpath('//Layer/MetadataURL/OnlineResource/@xlink:href',
+                namespaces=dict(xlink="http://www.w3.org/1999/xlink"))[0],
+            'http://some.url/')
+        eq_(xml.xpath('//Layer/MetadataURL/@type')[0],
+            'TC211')
+
         layer_names = set(xml.xpath('//Layer/Layer/Name/text()'))
-        expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
-            'wms_cache_transparent', 'wms_merge', 'tms_cache', 'wms_cache_multi',
-            'wms_cache_link_single', 'wms_cache_110'])
+        expected_names = set(['direct_fwd_params', 'direct', 'wms_cache',
+            'wms_cache_100', 'wms_cache_130', 'wms_cache_transparent',
+            'wms_merge', 'tms_cache', 'wms_cache_multi',
+            'wms_cache_link_single', 'wms_cache_110', 'watermark_cache'])
         eq_(layer_names, expected_names)
         assert validate_with_dtd(xml, dtd_name='wms/1.1.1/WMS_MS_Capabilities.dtd')
-    
+
     def test_invalid_layer(self):
         self.common_map_req.params['layers'] = 'invalid'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'application/vnd.ogc.se_xml')
         is_111_exception(resp.lxml, 'unknown layer: invalid', 'LayerNotDefined')
-    
+
     def test_invalid_layer_img_exception(self):
         self.common_map_req.params['layers'] = 'invalid'
         self.common_map_req.params['exceptions'] = 'application/vnd.ogc.se_inimage'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         assert is_png(StringIO(resp.body))
-    
+
     def test_invalid_format(self):
         self.common_map_req.params['format'] = 'image/ascii'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'application/vnd.ogc.se_xml')
-        is_111_exception(resp.lxml, 'unsupported image format: image/ascii', 
+        is_111_exception(resp.lxml, 'unsupported image format: image/ascii',
                          'InvalidFormat')
-    
+
     def test_invalid_format_img_exception(self):
         self.common_map_req.params['format'] = 'image/ascii'
         self.common_map_req.params['exceptions'] = 'application/vnd.ogc.se_inimage'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         assert is_png(StringIO(resp.body))
-    
+
     def test_invalid_format_options_img_exception(self):
         self.common_map_req.params['format'] = 'image/png; mode=12bit'
         self.common_map_req.params['exceptions'] = 'application/vnd.ogc.se_inimage'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         assert is_png(StringIO(resp.body))
-    
+
     def test_missing_format_img_exception(self):
         del self.common_map_req.params['format']
         self.common_map_req.params['exceptions'] = 'application/vnd.ogc.se_inimage'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         assert is_png(StringIO(resp.body))
-    
+
     def test_invalid_srs(self):
         self.common_map_req.params['srs'] = 'EPSG:1234'
         resp = self.app.get(self.common_map_req)
@@ -188,7 +197,7 @@ class TestWMS111(WMSTest):
         # is xml, even if inimage was requested
         eq_(resp.content_type, 'application/vnd.ogc.se_xml')
         is_111_exception(resp.lxml, 'image size too large')
-    
+
     def test_get_map_default_style(self):
         self.common_map_req.params['styles'] = 'default'
         resp = self.app.get(self.common_map_req)
@@ -196,7 +205,7 @@ class TestWMS111(WMSTest):
         data = StringIO(resp.body)
         assert is_png(data)
         assert Image.open(data).mode == 'RGB'
-    
+
     def test_get_map_png(self):
         resp = self.app.get(self.common_map_req)
         assert 'Cache-Control' not in resp.headers
@@ -253,13 +262,13 @@ class TestWMS111(WMSTest):
         img = Image.open(data)
         eq_(img.mode, 'RGB')
         eq_(sorted(img.getcolors())[-1][1], (255, 0, 160))
-    
+
     def test_get_map_jpeg(self):
         self.common_map_req.params['format'] = 'image/jpeg'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/jpeg')
         assert is_jpeg(StringIO(resp.body))
-    
+
     def test_get_map_xml_exception(self):
         self.common_map_req.params['bbox'] = '0,0,90,90'
         resp = self.app.get(self.common_map_req)
@@ -268,7 +277,7 @@ class TestWMS111(WMSTest):
         eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code'), [])
         assert 'No response from URL' in xml.xpath('//ServiceException/text()')[0]
         assert validate_with_dtd(xml, 'wms/1.1.1/exception_1_1_1.dtd')
-    
+
     def test_direct_layer_error(self):
         self.common_map_req.params['layers'] = 'direct'
         resp = self.app.get(self.common_map_req)
@@ -280,9 +289,9 @@ class TestWMS111(WMSTest):
         #     xml.xpath('//ServiceException/text()')[0]
         assert 'No response from URL' in \
              xml.xpath('//ServiceException/text()')[0]
-        
+
         assert validate_with_dtd(xml, 'wms/1.1.1/exception_1_1_1.dtd')
-    
+
     def test_get_map(self):
         self.created_tiles.append('wms_cache_EPSG900913/01/000/000/001/000/000/001.jpeg')
         with tmp_image((256, 256), format='jpeg') as img:
@@ -296,7 +305,20 @@ class TestWMS111(WMSTest):
                 resp = self.app.get(self.common_map_req)
                 assert 35000 < int(resp.headers['Content-length']) < 75000
                 eq_(resp.content_type, 'image/png')
-    
+
+    def test_get_map_direct_fwd_params_layer(self):
+        img = create_tmp_image((200, 200), format='png')
+        expected_req = ({'path': r'/service?LAYERs=bar&SERVICE=WMS&FORMAT=image%2Fpng'
+                                    '&REQUEST=GetMap&HEIGHT=200&SRS=EPSG%3A4326&styles='
+                                    '&VERSION=1.1.1&BBOX=-180.0,0.0,0.0,80.0'
+                                    '&WIDTH=200&TIME=20041012'},
+                        {'body': img})
+        with mock_httpd(('localhost', 42423), [expected_req]):
+            self.common_map_req.params['layers'] = 'direct_fwd_params'
+            self.common_map_req.params['time'] = '20041012'
+            resp = self.app.get(self.common_map_req)
+            eq_(resp.content_type, 'image/png')
+
     def test_get_map_use_direct_from_level(self):
         with tmp_image((200, 200), format='png') as img:
             expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fpng'
@@ -311,7 +333,7 @@ class TestWMS111(WMSTest):
                 assert resp.body == img.read()
                 is_png(img)
                 eq_(resp.content_type, 'image/png')
-        
+
     def test_get_map_use_direct_from_level_with_transform(self):
         with tmp_image((200, 200), format='png') as img:
             expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fpng'
@@ -327,24 +349,24 @@ class TestWMS111(WMSTest):
                 assert resp.body != img.read()
                 is_png(img)
                 eq_(resp.content_type, 'image/png')
-    
+
     def test_get_map_invalid_bbox(self):
         # min x larger than max x
         url =  """/service?SERVICE=WMS&VERSION=1.1.1&REQUEST=GetMap&BBOX=7,2,-9,10&SRS=EPSG:4326&WIDTH=164&HEIGHT=388&LAYERS=wms_cache&STYLES=&FORMAT=image/png&TRANSPARENT=TRUE"""
         resp = self.app.get(url)
         is_111_exception(resp.lxml, 'invalid bbox 7,2,-9,10')
-    
+
     def test_get_map_invalid_bbox2(self):
         # broken bbox for the requested srs
         url =  """/service?SERVICE=WMS&VERSION=1.1.1&REQUEST=GetMap&BBOX=-72988843.697212,-255661507.634227,142741550.188860,255661507.634227&SRS=EPSG:25833&WIDTH=164&HEIGHT=388&LAYERS=wms_cache_100&STYLES=&FORMAT=image/png&TRANSPARENT=TRUE"""
         resp = self.app.get(url)
         is_111_exception(resp.lxml, 'Request too large or invalid BBOX.')
-    
+
     def test_get_map_broken_bbox(self):
         url = """/service?VERSION=1.1.11&REQUEST=GetMap&SRS=EPSG:31467&BBOX=-10000855.0573254,2847125.18913603,-9329367.42767611,4239924.78564583&WIDTH=130&HEIGHT=62&LAYERS=wms_cache&STYLES=&FORMAT=image/png&TRANSPARENT=TRUE"""
         resp = self.app.get(url)
         is_111_exception(resp.lxml, 'Could not transform BBOX: Invalid result.')
-        
+
     def test_get_map100(self):
         self.created_tiles.append('wms_cache_100_EPSG900913/01/000/000/001/000/000/001.jpeg')
         # request_format tiff, cache format jpeg, wms request in png
@@ -359,7 +381,7 @@ class TestWMS111(WMSTest):
                 self.common_map_req.params['layers'] = 'wms_cache_100'
                 resp = self.app.get(self.common_map_req)
                 eq_(resp.content_type, 'image/png')
-    
+
     def test_get_map130(self):
         self.created_tiles.append('wms_cache_130_EPSG900913/01/000/000/001/000/000/001.jpeg')
         with tmp_image((256, 256), format='jpeg') as img:
@@ -367,13 +389,13 @@ class TestWMS111(WMSTest):
                                       '&REQUEST=GetMap&HEIGHT=256&CRS=EPSG%3A900913&styles='
                                       '&VERSION=1.3.0&BBOX=0.0,0.0,20037508.3428,20037508.3428'
                                       '&WIDTH=256'},
-                            {'body': img.read(), 'headers': {'content-type': 'image/jgeg'}})
+                            {'body': img.read(), 'headers': {'content-type': 'image/jpeg'}})
             with mock_httpd(('localhost', 42423), [expected_req]):
                 self.common_map_req.params['bbox'] = '0,0,180,90'
                 self.common_map_req.params['layers'] = 'wms_cache_130'
                 resp = self.app.get(self.common_map_req)
                 eq_(resp.content_type, 'image/png')
-    
+
     def test_get_map130_axis_order(self):
         self.created_tiles.append('wms_cache_multi_EPSG4326/02/000/000/003/000/000/001.jpeg')
         with tmp_image((256, 256), format='jpeg') as img:
@@ -382,13 +404,13 @@ class TestWMS111(WMSTest):
                                       '&REQUEST=GetMap&HEIGHT=256&CRS=EPSG%3A4326&styles='
                                       '&VERSION=1.3.0&BBOX=0.0,90.0,90.0,180.0'
                                       '&WIDTH=256'},
-                            {'body': img, 'headers': {'content-type': 'image/jgeg'}}),]
+                            {'body': img, 'headers': {'content-type': 'image/jpeg'}}),]
             with mock_httpd(('localhost', 42423), expected_reqs):
                 self.common_map_req.params['bbox'] = '90,0,180,90'
                 self.common_map_req.params['layers'] = 'wms_cache_multi'
                 resp = self.app.get(self.common_map_req)
                 eq_(resp.content_type, 'image/png')
-    
+
     def test_get_featureinfo(self):
         expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fpng'
                                   '&REQUEST=GetFeatureInfo&HEIGHT=200&SRS=EPSG%3A900913'
@@ -407,7 +429,7 @@ class TestWMS111(WMSTest):
                                   '&styles=&VERSION=1.1.1'
                                   '&WIDTH=200&QUERY_LAYERS=foo,bar&X=14&Y=78'},
                         {'body': 'info', 'headers': {'content-type': 'text/plain'}})
-        
+
         # out fi point at x=10,y=20
         p_25832  = (3570269+10*(3643458 - 3570269)/200, 5540889+20*(5614078 - 5540889)/200)
         # the transformed fi point at x=10,y=22
@@ -418,7 +440,7 @@ class TestWMS111(WMSTest):
         # check with tolerance: pixel resolution is ~570 and x/y position is rounded to pizel
         assert abs(SRS(25832).transform_to(SRS(900913), p_25832)[0] - p_900913[0]) < 570/2
         assert abs(SRS(25832).transform_to(SRS(900913), p_25832)[1] - p_900913[1]) < 570/2
-        
+
         with mock_httpd(('localhost', 42423), [expected_req], bbox_aware_query_comparator=True):
             self.common_fi_req.params['bbox'] = '3570269,5540889,3643458,5614078'
             self.common_fi_req.params['srs'] = 'EPSG:25832'
@@ -439,7 +461,7 @@ class TestWMS111(WMSTest):
             resp = self.app.get(self.common_fi_req)
             eq_(resp.content_type, 'text/html')
             eq_(resp.body, 'info')
-    
+
     def test_get_featureinfo_130(self):
         expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fpng'
                                   '&REQUEST=GetFeatureInfo&HEIGHT=200&CRS=EPSG%3A900913'
@@ -452,7 +474,7 @@ class TestWMS111(WMSTest):
             resp = self.app.get(self.common_fi_req)
             eq_(resp.content_type, 'text/plain')
             eq_(resp.body, 'info')
-        
+
     def test_get_featureinfo_missing_params(self):
         expected_req = (
             {'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fpng'
@@ -466,13 +488,13 @@ class TestWMS111(WMSTest):
             resp = self.app.get(self.common_fi_req)
             eq_(resp.content_type, 'text/plain')
             eq_(resp.body, 'info')
-    
+
     def test_get_featureinfo_missing_params_strict(self):
         request_parser = self.app.app.handlers['service'].services['wms'].request_parser
         try:
             self.app.app.handlers['service'].services['wms'].request_parser = \
                 functools.partial(wms_request, strict=True)
-        
+
             del self.common_fi_req.params['format']
             del self.common_fi_req.params['styles']
             resp = self.app.get(self.common_fi_req)
@@ -482,7 +504,7 @@ class TestWMS111(WMSTest):
         finally:
             self.app.app.handlers['service'].services['wms'].request_parser = request_parser
             self.app.app.handlers['service'].request_parser = request_parser
-    
+
     def test_get_featureinfo_not_queryable(self):
         self.common_fi_req.params['query_layers'] = 'tms_cache'
         self.common_fi_req.params['exceptions'] = 'application/vnd.ogc.se_xml'
@@ -492,14 +514,14 @@ class TestWMS111(WMSTest):
         eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code'), [])
         assert 'tms_cache is not queryable' in xml.xpath('//ServiceException/text()')[0]
         assert validate_with_dtd(xml, 'wms/1.1.1/exception_1_1_1.dtd')
-    
+
 
 class TestWMS110(WMSTest):
     def setup(self):
         WMSTest.setup(self)
-        self.common_req = WMS110MapRequest(url='/service?', param=dict(service='WMS', 
+        self.common_req = WMS110MapRequest(url='/service?', param=dict(service='WMS',
              version='1.1.0'))
-        self.common_map_req = WMS110MapRequest(url='/service?', param=dict(service='WMS', 
+        self.common_map_req = WMS110MapRequest(url='/service?', param=dict(service='WMS',
              version='1.1.0', bbox='-180,0,0,80', width='200', height='200',
              layers='wms_cache', srs='EPSG:4326', format='image/png',
              styles='', request='GetMap'))
@@ -507,7 +529,7 @@ class TestWMS110(WMSTest):
             param=dict(x='10', y='20', width='200', height='200', layers='wms_cache',
                        format='image/png', query_layers='wms_cache_110', styles='',
                        bbox='1000,400,2000,1400', srs='EPSG:900913'))
-                       
+
     def test_wms_capabilities(self):
         req = WMS110CapabilitiesRequest(url='/service?').copy_with_request_params(self.common_req)
         resp = self.app.get(req)
@@ -516,18 +538,19 @@ class TestWMS110(WMSTest):
         eq_(xml.xpath('//GetMap//OnlineResource/@xlink:href',
                       namespaces=dict(xlink="http://www.w3.org/1999/xlink"))[0],
             'http://localhost/service?')
-        
+
         llbox = xml.xpath('//Capability/Layer/LatLonBoundingBox')[0]
         # some clients don't like 90deg north/south
         assert_almost_equal(float(llbox.attrib['miny']), -89.999999, 6)
         assert_almost_equal(float(llbox.attrib['maxy']), 89.999999, 6)
         assert_almost_equal(float(llbox.attrib['minx']), -180.0, 6)
         assert_almost_equal(float(llbox.attrib['maxx']), 180.0, 6)
-        
+
         layer_names = set(xml.xpath('//Layer/Layer/Name/text()'))
-        expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
-            'wms_cache_transparent', 'wms_merge', 'tms_cache', 'wms_cache_multi',
-            'wms_cache_link_single', 'wms_cache_110'])
+        expected_names = set(['direct_fwd_params', 'direct', 'wms_cache',
+            'wms_cache_100', 'wms_cache_130', 'wms_cache_transparent',
+            'wms_merge', 'tms_cache', 'wms_cache_multi',
+            'wms_cache_link_single', 'wms_cache_110', 'watermark_cache'])
         eq_(layer_names, expected_names)
         assert validate_with_dtd(xml, dtd_name='wms/1.1.0/capabilities_1_1_0.dtd')
 
@@ -550,14 +573,14 @@ class TestWMS110(WMSTest):
         eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code')[0], 'InvalidFormat')
         eq_(xml.xpath('//ServiceException/text()')[0], 'unsupported image format: image/ascii')
         assert validate_with_dtd(xml, dtd_name='wms/1.1.0/exception_1_1_0.dtd')
-    
+
     def test_invalid_format_img_exception(self):
         self.common_map_req.params['format'] = 'image/ascii'
         self.common_map_req.params['exceptions'] = 'application/vnd.ogc.se_inimage'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         assert is_png(StringIO(resp.body))
-    
+
     def test_missing_format_img_exception(self):
         del self.common_map_req.params['format']
         self.common_map_req.params['exceptions'] = 'application/vnd.ogc.se_inimage'
@@ -574,20 +597,20 @@ class TestWMS110(WMSTest):
         eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code')[0], 'InvalidSRS')
         eq_(xml.xpath('//ServiceException/text()')[0], 'unsupported srs: EPSG:1234')
         assert validate_with_dtd(xml, dtd_name='wms/1.1.0/exception_1_1_0.dtd')
-        
+
     def test_get_map_png(self):
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         data = StringIO(resp.body)
         assert is_png(data)
         assert Image.open(data).mode == 'RGB'
-    
+
     def test_get_map_jpeg(self):
         self.common_map_req.params['format'] = 'image/jpeg'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/jpeg')
         assert is_jpeg(StringIO(resp.body))
-    
+
     def test_get_map_xml_exception(self):
         self.common_map_req.params['bbox'] = '0,0,90,90'
         resp = self.app.get(self.common_map_req)
@@ -610,7 +633,7 @@ class TestWMS110(WMSTest):
                 resp = self.app.get(self.common_map_req)
                 assert 35000 < int(resp.headers['Content-length']) < 75000
                 eq_(resp.content_type, 'image/png')
-    
+
     def test_get_map_110(self):
         self.created_tiles.append('wms_cache_110_EPSG900913/01/000/000/001/000/000/001.jpeg')
         with tmp_image((256, 256), format='jpeg') as img:
@@ -625,7 +648,7 @@ class TestWMS110(WMSTest):
                 resp = self.app.get(self.common_map_req)
                 assert 35000 < int(resp.headers['Content-length']) < 75000
                 eq_(resp.content_type, 'image/png')
-    
+
     def test_get_featureinfo(self):
         expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fpng'
                                   '&REQUEST=GetFeatureInfo&HEIGHT=200&SRS=EPSG%3A900913'
@@ -636,7 +659,7 @@ class TestWMS110(WMSTest):
             resp = self.app.get(self.common_fi_req)
             eq_(resp.content_type, 'text/plain')
             eq_(resp.body, 'info')
-            
+
     def test_get_featureinfo_not_queryable(self):
         self.common_fi_req.params['query_layers'] = 'tms_cache'
         self.common_fi_req.params['exceptions'] = 'application/vnd.ogc.se_xml'
@@ -646,7 +669,7 @@ class TestWMS110(WMSTest):
         eq_(xml.xpath('/ServiceExceptionReport/ServiceException/@code'), [])
         assert 'tms_cache is not queryable' in xml.xpath('//ServiceException/text()')[0]
         assert validate_with_dtd(xml, 'wms/1.1.0/exception_1_1_0.dtd')
-    
+
 
 class TestWMS100(WMSTest):
     def setup(self):
@@ -660,7 +683,7 @@ class TestWMS100(WMSTest):
             param=dict(x='10', y='20', width='200', height='200', layers='wms_cache_100',
                        format='image/png', query_layers='wms_cache_100', styles='',
                        bbox='1000,400,2000,1400', srs='EPSG:900913'))
-        
+
     def test_wms_capabilities(self):
         req = WMS100CapabilitiesRequest(url='/service?').copy_with_request_params(self.common_req)
         resp = self.app.get(req)
@@ -669,14 +692,15 @@ class TestWMS100(WMSTest):
         eq_(xml.xpath('/WMT_MS_Capabilities/Service/Title/text()')[0],
             u'MapProxy test fixture \u2603')
         layer_names = set(xml.xpath('//Layer/Layer/Name/text()'))
-        expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
-            'wms_cache_transparent', 'wms_merge', 'tms_cache', 'wms_cache_multi',
-            'wms_cache_link_single', 'wms_cache_110'])
+        expected_names = set(['direct_fwd_params', 'direct', 'wms_cache',
+            'wms_cache_100', 'wms_cache_130', 'wms_cache_transparent',
+            'wms_merge', 'tms_cache', 'wms_cache_multi',
+            'wms_cache_link_single', 'wms_cache_110', 'watermark_cache'])
         eq_(layer_names, expected_names)
         #TODO srs
         assert validate_with_dtd(xml, dtd_name='wms/1.0.0/capabilities_1_0_0.dtd')
-        
-    
+
+
     def test_invalid_layer(self):
         self.common_map_req.params['layers'] = 'invalid'
         resp = self.app.get(self.common_map_req)
@@ -684,7 +708,7 @@ class TestWMS100(WMSTest):
         xml = resp.lxml
         eq_(xml.xpath('/WMTException/@version')[0], '1.0.0')
         eq_(xml.xpath('//WMTException/text()')[0].strip(), 'unknown layer: invalid')
-    
+
     def test_invalid_format(self):
         self.common_map_req.params['format'] = 'image/ascii'
         resp = self.app.get(self.common_map_req)
@@ -693,35 +717,35 @@ class TestWMS100(WMSTest):
         eq_(xml.xpath('/WMTException/@version')[0], '1.0.0')
         eq_(xml.xpath('//WMTException/text()')[0].strip(),
                       'unsupported image format: image/ascii')
-    
+
     def test_invalid_format_img_exception(self):
         self.common_map_req.params['format'] = 'image/ascii'
         self.common_map_req.params['exceptions'] = 'INIMAGE'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         assert is_png(StringIO(resp.body))
-    
+
     def test_missing_format_img_exception(self):
         del self.common_map_req.params['format']
         self.common_map_req.params['exceptions'] = 'INIMAGE'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         assert is_png(StringIO(resp.body))
-    
+
     def test_invalid_srs(self):
         self.common_map_req.params['srs'] = 'EPSG:1234'
         print self.common_map_req.complete_url
         resp = self.app.get(self.common_map_req.complete_url)
         xml = resp.lxml
         eq_(xml.xpath('//WMTException/text()')[0].strip(), 'unsupported srs: EPSG:1234')
-    
+
     def test_get_map_png(self):
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         data = StringIO(resp.body)
         assert is_png(data)
         eq_(Image.open(data).mode, 'RGB')
-        
+
     def test_get_map_png_transparent_paletted(self):
         try:
             base_config().image.paletted = True
@@ -733,19 +757,19 @@ class TestWMS100(WMSTest):
             assert Image.open(data).mode == 'P'
         finally:
             base_config().image.paletted = False
-            
+
     def test_get_map_jpeg(self):
         self.common_map_req.params['format'] = 'image/jpeg'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/jpeg')
         assert is_jpeg(StringIO(resp.body))
-    
+
     def test_get_map_xml_exception(self):
          self.common_map_req.params['bbox'] = '0,0,90,90'
          resp = self.app.get(self.common_map_req)
          xml = resp.lxml
          assert 'No response from URL' in xml.xpath('//WMTException/text()')[0]
-    
+
     def test_get_map(self):
         self.created_tiles.append('wms_cache_EPSG900913/01/000/000/001/000/000/001.jpeg')
         with tmp_image((256, 256), format='jpeg') as img:
@@ -753,12 +777,12 @@ class TestWMS100(WMSTest):
                                       '&REQUEST=GetMap&HEIGHT=256&SRS=EPSG%3A900913&styles='
                                       '&VERSION=1.1.1&BBOX=0.0,0.0,20037508.3428,20037508.3428'
                                       '&WIDTH=256'},
-                            {'body': img.read(), 'headers': {'content-type': 'image/jgeg'}})
+                            {'body': img.read(), 'headers': {'content-type': 'image/jpeg'}})
             with mock_httpd(('localhost', 42423), [expected_req]):
                 self.common_map_req.params['bbox'] = '0,0,180,90'
                 resp = self.app.get(self.common_map_req)
                 eq_(resp.content_type, 'image/png')
-    
+
     def test_get_featureinfo(self):
         expected_req = ({'path': r'/service?LAYERs=foo,bar&FORMAT=image%2Fpng'
                                   '&REQUEST=feature_info&HEIGHT=200&SRS=EPSG%3A900913'
@@ -769,7 +793,7 @@ class TestWMS100(WMSTest):
             resp = self.app.get(self.common_fi_req)
             eq_(resp.content_type, 'text/plain')
             eq_(resp.body, 'info')
-    
+
     def test_get_featureinfo_not_queryable(self):
         self.common_fi_req.params['query_layers'] = 'tms_cache'
         self.common_fi_req.params['exceptions'] = 'application/vnd.ogc.se_xml'
@@ -791,9 +815,9 @@ eq_xpath_wms130 = functools.partial(eq_xpath, namespaces=ns130)
 class TestWMS130(WMSTest):
     def setup(self):
         WMSTest.setup(self)
-        self.common_req = WMS130MapRequest(url='/service?', param=dict(service='WMS', 
+        self.common_req = WMS130MapRequest(url='/service?', param=dict(service='WMS',
              version='1.3.0'))
-        self.common_map_req = WMS130MapRequest(url='/service?', param=dict(service='WMS', 
+        self.common_map_req = WMS130MapRequest(url='/service?', param=dict(service='WMS',
              version='1.3.0', bbox='0,-180,80,0', width='200', height='200',
              layers='wms_cache', crs='EPSG:4326', format='image/png',
              styles='', request='GetMap'))
@@ -816,12 +840,13 @@ class TestWMS130(WMSTest):
 
         layer_names = set(xml.xpath('//wms:Layer/wms:Layer/wms:Name/text()',
                                     namespaces=ns130))
-        expected_names = set(['direct', 'wms_cache', 'wms_cache_100', 'wms_cache_130',
-            'wms_cache_transparent', 'wms_merge', 'tms_cache', 'wms_cache_multi',
-            'wms_cache_link_single', 'wms_cache_110'])
+        expected_names = set(['direct_fwd_params', 'direct', 'wms_cache',
+            'wms_cache_100', 'wms_cache_130', 'wms_cache_transparent',
+            'wms_merge', 'tms_cache', 'wms_cache_multi',
+            'wms_cache_link_single', 'wms_cache_110', 'watermark_cache'])
         eq_(layer_names, expected_names)
         assert is_130_capa(xml)
-    
+
     def test_invalid_layer(self):
         self.common_map_req.params['layers'] = 'invalid'
         resp = self.app.get(self.common_map_req)
@@ -832,7 +857,7 @@ class TestWMS130(WMSTest):
             'LayerNotDefined')
         eq_xpath_wms130(xml, '//ogc:ServiceException/text()', 'unknown layer: invalid')
         assert validate_with_xsd(xml, xsd_name='wms/1.3.0/exceptions_1_3_0.xsd')
-    
+
     def test_invalid_format(self):
         self.common_map_req.params['format'] = 'image/ascii'
         resp = self.app.get(self.common_map_req)
@@ -843,25 +868,25 @@ class TestWMS130(WMSTest):
             'InvalidFormat')
         eq_xpath_wms130(xml, '//ogc:ServiceException/text()', 'unsupported image format: image/ascii')
         assert validate_with_xsd(xml, xsd_name='wms/1.3.0/exceptions_1_3_0.xsd')
-    
+
     def test_invalid_format_img_exception(self):
         self.common_map_req.params['format'] = 'image/ascii'
         self.common_map_req.params['exceptions'] = 'application/vnd.ogc.se_inimage'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         assert is_png(StringIO(resp.body))
-    
+
     def test_missing_format_img_exception(self):
         del self.common_map_req.params['format']
         self.common_map_req.params['exceptions'] = 'application/vnd.ogc.se_inimage'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         assert is_png(StringIO(resp.body))
-    
+
     def test_invalid_srs(self):
         self.common_map_req.params['srs'] = 'EPSG:1234'
         self.common_map_req.params['exceptions'] = 'text/xml'
-        
+
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'text/xml')
         xml = resp.lxml
@@ -869,20 +894,20 @@ class TestWMS130(WMSTest):
             'InvalidCRS')
         eq_xpath_wms130(xml, '//ogc:ServiceException/text()', 'unsupported crs: EPSG:1234')
         assert validate_with_xsd(xml, xsd_name='wms/1.3.0/exceptions_1_3_0.xsd')
-    
+
     def test_get_map_png(self):
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/png')
         data = StringIO(resp.body)
         assert is_png(data)
         assert Image.open(data).mode == 'RGB'
-        
+
     def test_get_map_jpeg(self):
         self.common_map_req.params['format'] = 'image/jpeg'
         resp = self.app.get(self.common_map_req)
         eq_(resp.content_type, 'image/jpeg')
         assert is_jpeg(StringIO(resp.body))
-    
+
     def test_get_map_xml_exception(self):
         self.common_map_req.params['bbox'] = '0,0,90,90'
         resp = self.app.get(self.common_map_req)
@@ -892,7 +917,7 @@ class TestWMS130(WMSTest):
         assert ('No response from URL' in
             xml.xpath('//ogc:ServiceException/text()', namespaces=ns130)[0])
         assert validate_with_xsd(xml, xsd_name='wms/1.3.0/exceptions_1_3_0.xsd')
-    
+
     def test_get_map(self):
         self.created_tiles.append('wms_cache_EPSG900913/01/000/000/001/000/000/001.jpeg')
         with tmp_image((256, 256), format='jpeg') as img:
@@ -900,12 +925,12 @@ class TestWMS130(WMSTest):
                                       '&REQUEST=GetMap&HEIGHT=256&SRS=EPSG%3A900913&styles='
                                       '&VERSION=1.1.1&BBOX=0.0,0.0,20037508.3428,20037508.3428'
                                       '&WIDTH=256'},
-                            {'body': img.read(), 'headers': {'content-type': 'image/jgeg'}})
+                            {'body': img.read(), 'headers': {'content-type': 'image/jpeg'}})
             with mock_httpd(('localhost', 42423), [expected_req]):
                 self.common_map_req.params['bbox'] = '0,0,180,90' #internal axis-order
                 resp = self.app.get(self.common_map_req)
                 eq_(resp.content_type, 'image/png')
-    
+
     def test_get_featureinfo(self):
         expected_req = ({'path': r'/service?LAYERs=foo,bar&SERVICE=WMS&FORMAT=image%2Fpng'
                                   '&REQUEST=GetFeatureInfo&HEIGHT=200&CRS=EPSG%3A900913'
@@ -929,17 +954,17 @@ class TestWMS130(WMSTest):
             resp = self.app.get(self.common_fi_req)
             eq_(resp.content_type, 'text/plain')
             eq_(resp.body, 'info')
-    
+
 
 if sys.platform != 'win32':
     class TestWMSLinkSingleColorImages(WMSTest):
         def setup(self):
             WMSTest.setup(self)
-            self.common_map_req = WMS111MapRequest(url='/service?', param=dict(service='WMS', 
+            self.common_map_req = WMS111MapRequest(url='/service?', param=dict(service='WMS',
                  version='1.1.1', bbox='-180,0,0,80', width='200', height='200',
                  layers='wms_cache_link_single', srs='EPSG:4326', format='image/jpeg',
                  styles='', request='GetMap'))
-    
+
         def test_get_map(self):
             link_name = 'wms_cache_link_single_EPSG900913/01/000/000/001/000/000/001.png'
             real_name = 'wms_cache_link_single_EPSG900913/single_color_tiles/fe00a0.png'
@@ -955,14 +980,14 @@ if sys.platform != 'win32':
                     self.common_map_req.params['bbox'] = '0,0,180,90'
                     resp = self.app.get(self.common_map_req)
                     eq_(resp.content_type, 'image/jpeg')
-            
+
                 base_dir = base_config().cache.base_dir
                 single_loc = os.path.join(base_dir, real_name)
                 tile_loc = os.path.join(base_dir, link_name)
                 assert os.path.exists(single_loc)
                 assert os.path.islink(tile_loc)
-            
+
                 self.common_map_req.params['format'] = 'image/png'
                 resp = self.app.get(self.common_map_req)
                 eq_(resp.content_type, 'image/png')
-            
\ No newline at end of file
+
